#include "gui_create_torrent.h"
#include "logger.h"

gui_create_torrent::gui_create_torrent() :
  lbl_save_path("Save path:"),
  btn_start_seeding("Start seeding immediately"),
  lbl_tracker_url("Tracker URL")
{
    set_title("Create torrent");
    set_border_width(10);
    Gtk::Image open_img(Gtk::Stock::OPEN, Gtk::ICON_SIZE_BUTTON);

    this->property_default_width().set_value(400);

    /* All tabs, Cancel */
    cancel_btn = new Gtk::Button(Gtk::Stock::CANCEL);
    cancel_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &gui_create_torrent::cancel));

    /* All tabs, Torrent path */
    select_torrent_path_btn = new Gtk::Button(Gtk::Stock::OPEN);
    select_torrent_path_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &gui_create_torrent::select_torrent_path));

    /* All tabs, Create torrent */
    create_btn = new Gtk::Button(Gtk::Stock::SAVE);
    create_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &gui_create_torrent::create));

    /* Files tab, Add file */
    file_add_btn  = new Gtk::Button(Gtk::Stock::ADD);
    file_add_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &gui_create_torrent::add_file_dialog));

    /* Trackers tab, Add tracker */
    tracker_add_btn  = new Gtk::Button(Gtk::Stock::ADD);
    tracker_add_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &gui_create_torrent::add_tracker));

    box.pack_start(tabs, 0, 0, 0);
    box.pack_start(m_path_box, Gtk::PACK_SHRINK);
    box.pack_start(m_btn_box, Gtk::PACK_SHRINK);

    m_btn_box.pack_start(lbl_status, Gtk::PACK_SHRINK);
    m_btn_box.pack_end(*cancel_btn, Gtk::PACK_SHRINK);
    m_btn_box.pack_end(*create_btn, Gtk::PACK_SHRINK);
    m_btn_box.pack_end(btn_start_seeding, Gtk::PACK_SHRINK);

    m_path_box.pack_end(*select_torrent_path_btn, Gtk::PACK_SHRINK);
    m_path_box.pack_end(ent_save_path);
    m_path_box.pack_start(lbl_save_path, 0, 0, 0);

    tabs.append_page(files_page, "Files");
    tabs.append_page(trackers_page, "Trackers");

    /* Files tab */
    frefTreeModel = Gtk::ListStore::create(fcol);
    ftreeview.set_model(frefTreeModel);
    ftreeview.set_size_request(400, 250);

    Gtk::TreeView::Column *fcol_file = 
        Gtk::manage(new Gtk::TreeView::Column("File"));

    Gtk::CellRendererText *fcell_file = 
        Gtk::manage(new Gtk::CellRendererText);

    fcol_file->pack_start(*fcell_file);

    ftreeview.append_column(*fcol_file);
    fcol_file->add_attribute(fcell_file->property_text(), fcol.name);

    f_top_box.pack_end(*file_add_btn, Gtk::PACK_SHRINK);

    f_scrolledwindow.add(ftreeview);
    f_scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    files_page.pack_start(f_top_box);
    files_page.pack_start(f_scrolledwindow);

    /* Trackers tab */
    trefTreeModel = Gtk::ListStore::create(tcol);
    ttreeview.set_model(trefTreeModel);
    ttreeview.set_size_request(400, 250);

    Gtk::TreeView::Column *tcol_url = 
        Gtk::manage(new Gtk::TreeView::Column("URL"));

    Gtk::CellRendererText *tcell_url = 
        Gtk::manage(new Gtk::CellRendererText);

    tcol_url->pack_start(*tcell_url);

    ttreeview.append_column(*tcol_url);
    tcol_url->add_attribute(tcell_url->property_text(), tcol.url);

    t_top_box.pack_start(lbl_tracker_url, Gtk::PACK_SHRINK);
    t_top_box.pack_start(ent_tracker_url, Gtk::PACK_SHRINK);
    t_top_box.pack_start(*tracker_add_btn, Gtk::PACK_SHRINK);

    t_scrolledwindow.add(ttreeview);
    t_scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    trackers_page.pack_start(t_top_box);
    trackers_page.pack_start(t_scrolledwindow);

    add(box);
    show_all();

#if LIBTORRENT_VERSION_MINOR < 14
    lbl_status.set_text("Your version of libtorrent might not be able to create proper torrents");
#endif
}

/** 
 * Destroy the create torrent dialog
 **/
gui_create_torrent::~gui_create_torrent()
{
    delete cancel_btn;
    delete select_torrent_path_btn;
    delete create_btn;
    delete file_add_btn;
    delete tracker_add_btn;
    hide();
}

/** 
 * Create the torrent file
 **/
void
gui_create_torrent::create()
{
    std::vector<Glib::ustring> files;
    std::vector<Glib::ustring> trackers;
    char root_path[512];
    unsigned int len, i, c, x = 0, min_c = 0;
    int ret;
    Glib::ustring torrent_path = ent_save_path.get_text();
    typedef Gtk::TreeModel::Children type_children;

    /* Loop through all files and add them to the files vector */
    type_children fchildren = frefTreeModel->children();
    for (type_children::iterator iter = fchildren.begin();
            iter != fchildren.end(); ++iter) {
        Glib::ustring tmp((*iter)[fcol.name]);
        len = strlen(tmp.c_str());

        /* count number of frontslashes to determine
         * which path is the root path */
        for (i = 0, c = 0; i < len; i++) {
            if (tmp[i] == '/') {
                c++;
                x = i;
            }
        }
        if (!min_c || c < min_c) {
            ret = snprintf(root_path, 512, "%s", tmp.substr(0,x).c_str());
            if (ret == -1 || root_path == NULL) {
                fprintf(stderr, "out of mem");
                abort();
            }
            min_c = c;
        }

        files.push_back(tmp);
    }

    /* Loop through all trackers and add them to the trackers vector */
    type_children tchildren = trefTreeModel->children();
    for (type_children::iterator iter = tchildren.begin();
            iter != tchildren.end(); ++iter) {
        trackers.push_back((*iter)[tcol.url]);
    }

    try {
        g_torrent->create_torrent(files, trackers, torrent_path.c_str());
    } catch (std::exception &e) {
        g_logger->errorf("exception caught: %s", e.what());
        lbl_status.set_text(e.what());
        return;
    }

#ifndef WIN32
    if (btn_start_seeding.get_active()) {
        char *str;
        ret = asprintf(&str, "add:%s;%s", torrent_path.c_str(), root_path);
        if (ret == -1 || str == NULL) {
            fprintf(stderr, "out of mem");
            abort();
        }
        g_logger->messagef("add:%s;%s", torrent_path.c_str(), root_path);
        pipe_append(str);
        free(str);
    }
#endif

    hide();
}

/** 
 * Destroy the create_torrent dialog
 **/
void
gui_create_torrent::cancel()
{
    hide();
}

/** 
 * Display a dialog that lets you pick a file to add to the torrent
 **/
void
gui_create_torrent::add_file_dialog()
{
    Gtk::FileChooserDialog dialog("Please choose a file",
            Gtk::FILE_CHOOSER_ACTION_OPEN);
    dialog.set_transient_for(*this);
    dialog.set_select_multiple();

    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

    Gtk::FileFilter filter_any;
    filter_any.set_name("Any files");
    filter_any.add_pattern("*");
    dialog.add_filter(filter_any);

    int result = dialog.run();

    switch(result) {
        case(Gtk::RESPONSE_OK):
        {
            std::vector<Glib::ustring> files = dialog.get_filenames();
            std::vector<Glib::ustring>::iterator i;
            for (i = files.begin(); i != files.end(); ++i) {
                /* TODO: look for duplicates */
                Gtk::TreeModel::Row row = *(frefTreeModel->append());

                row[fcol.name] = *i;
            }

            break;
        }
    }
}

/** 
 * Add the value of ent_tracker_url to the torrent, default protocol is http
 **/
void
gui_create_torrent::add_tracker()
{
    Glib::ustring url = ent_tracker_url.get_text();
    if (url.length() < 1)
        return;
    ent_tracker_url.set_text("");

    char *p;
    if (!(p = (char*)malloc(url.length()+1)))
        fprintf(stderr, "out of mem");

    sprintf(p, "%s", url.c_str());
    while (isalnum(*p))
        p++;

    if (*p != ':')
        url = "http://"+url;

    Gtk::TreeModel::Row row = *(trefTreeModel->append());

    row[tcol.url] = url;
}

/** 
 * Display a dialog which lets you chose the save path of your torrent
 **/
void
gui_create_torrent::select_torrent_path()
{
    Gtk::FileChooserDialog dialog("Please choose a file path",
            Gtk::FILE_CHOOSER_ACTION_SAVE);
    dialog.set_transient_for(*this);

    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

    int result = dialog.run();

    switch(result) {
        case(Gtk::RESPONSE_OK):
        {
            Glib::ustring path = dialog.get_filename();
            ent_save_path.set_text(path);

            break;
        }
    }
}
