#include "logger.h"
#include "tools.h"

#include <iostream>
#include <gtkmm.h>
#include <gtkmm/dialog.h>

Logger *g_logger;

#ifdef WIN32
int vasprintf(char **strp, const char *fmt, va_list ap) {
    int len = vsnprintf(NULL, 0, fmt, ap) + 1;
    char *res = (char *)malloc (len);
    if (res == NULL)
        return -1;
    *strp = res;
    return vsprintf(res, fmt, ap);
}
#endif

void Logger::
output(Gtk::MessageType type, const char *str, bool graphical)
{
    /** 
     * Regardless of whether the logger is enabled or not, we should
     * still be able to print out messages to the console.
     **/
    switch (type) {
        case Gtk::MESSAGE_WARNING:
            std::cerr << "[W] " << str << '\n';
            break;

        case Gtk::MESSAGE_ERROR:
            std::cerr << "[E] " << str << '\n';
            break;

        default:
            std::cout << "[I] " << str << '\n';
    }

    if (this->enabled) {
        this->lbl_status_msg->set_text(Glib::locale_to_utf8(str));

        if (graphical) {
            Gtk::MessageDialog msg(*this, str, false, 
                    type, Gtk::BUTTONS_OK);
            msg.run();
        }
    }
}

void Logger::
update_status(float down, float up)
{
    const char *c_down = fix_rate(down);
    const char *c_up   = fix_rate(up);
    char tmp[32];

    sprintf(tmp, "Down: %s", c_down);
    this->lbl_status_download->set_text(tmp);

    sprintf(tmp, "Up: %s", c_up);
    this->lbl_status_upload->set_text(tmp);
}

void
Logger::errorf(const char *err, ...)
{
    char *buf;

    va_list va;
    va_start(va, err);

    if (vasprintf(&buf, err, va) == -1) {
        fprintf(stderr, "out of mem\n");
        abort();
    }
    output(Gtk::MESSAGE_ERROR, buf, false);
    free(buf);
    va_end(va);
}

void
Logger::warningf(const char *warn, ...)
{
    char *buf;

    va_list va;
    va_start(va, warn);

    if (vasprintf(&buf, warn, va) == -1) {
        fprintf(stderr, "out of mem\n");
        abort();
    }
    output(Gtk::MESSAGE_WARNING, buf, false);
    free(buf);
    va_end(va);
}

void
Logger::messagef(const char *msg, ...)
{
    char *buf;

    va_list va;
    va_start(va, msg);

    if (vasprintf(&buf, msg, va) == -1) {
        fprintf(stderr, "out of mem\n");
        abort();
    }
    output(Gtk::MESSAGE_INFO, buf, false);
    free(buf);
    va_end(va);
}
