#include <gtkmm/main.h>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <errno.h>
#include <signal.h>

#include "window.h"

class Settings;
class Torrent;
class Logger;

static void sigint_handler(int sig);

MainWindow *window;

int
main(int argc, char **argv)
{
    const char *pipename;
    const char *pid;

    Gtk::Main kit(argc, argv);

    if (!(g_logger = new Logger())) {
        fprintf(stderr, "out of mem\n");
        return 1;
    }
    if (!(g_settings = new Settings())) {
        fprintf(stderr, "out of mem\n");
        return 1;
    }

    pipename = g_settings->pipe_file.c_str();
    pid      = g_settings->pid_file.c_str();

#ifndef WIN32
    if (!access(pid, F_OK)) {
        FILE *fp = fopen(pid, "r");
        if (fp) {
            int pid_n;
            bool running;
            char tmp[32];
            if (fscanf(fp, "%d", &pid_n) == -1)
                return 1;
            fclose(fp);
            if (sprintf(tmp, "/proc/%d", pid_n) == -1)
                return 1;
            if (access(tmp, F_OK) != 0) 
                unlink(pid);
            else {
                printf("torium running with pid %d\n", pid_n);
                running = true;

                if (sprintf(tmp, "/proc/%d/stat", pid_n) == -1)
                    return 1;
                if (access(tmp, F_OK) == 0) {
                    std::fstream fh(tmp, std::ios::in);
                    char in[256];

                    int start, end;
                    while (!fh.eof()) {
                        fh.getline(in, 256);
                        if (*in == '\0')
                            break;
                        std::string s(in);
                        start = s.find('(');
                        end   = s.find(')');
                        if (s.substr(start+1, end-start-1) != "torium") {
                            printf("lol just kidding\n"
                                   "%s is running with pid %d\n",
                                   s.substr(start+1, end-start-1).c_str(), pid_n);
                            running = false;
                        }
                        break;
                    }
                }
                /* torium is running, we must send our torrents through its
                 * opened  pipe */
                if (running && !access(pipename, F_OK)) {
                    if(argc >= 2) {
                        FILE *stream;
                        char *tmp;
                        int x = argc-1;
                        while(x--) {
                            printf("adding %s\n", argv[x+1]);
                            if(tmp) {
                                if (asprintf(&tmp, "%s\nadd:%s",
                                            tmp, argv[x+1]) == -1) {
                                    continue;
                                }
                            } else {
                                if (asprintf(&tmp, "add:%s", argv[x+1]) == -1)
                                    continue;
                            }
                        }
                        stream = fopen(pipename, "a");
                        fprintf(stream, "%s ", tmp);
                        free(tmp);
                        fclose(stream);
                    }

                    delete g_settings;
                    delete g_logger;

                    return 0;
                }
            }
        }
    }
#endif
    /** 
     * g_logger should not be enabled before the whole gui is displayed,
     * otherwise we run the risk of attempting to set the statusbar label text
     * before it's actually initialized
     *
     * g_logger->enable();
     **/

    if (!(g_torrent = new Torrent())) {
        fprintf(stderr, "out of mem\n");
        return 1;
    }

    /* loop through all arguments */
    struct options {
        int id;
        const char *longopt;
        bool requires_argument;
        const int shortopt;
    };

    static options opts[] = {
    /*   id  longopt          requires_argument  shortopt */
        {0,  "download-dir",  true,              'd'},
        {1,  "geom",          true,              'g'},
        {2,  "pos",           true,              'p'},
        {3,  "add",           true,              'a'}
    };

    int win_x=-1, win_y=-1;
    int win_width=-1, win_height=-1;
    int i, x, id=-1, sz = sizeof(opts)/sizeof(struct options);
    bool found=false, arg=false;
    char *k  = 0;
    char *v  = 0;
    char *ke = 0;
    std::vector<const char*> torrents;

    if (argc > 1) {
        /* something needs to be done to the additional arguments */
        for (i = 1; i < argc; ++i, found=false, arg=false, id=-1) {
            /* The first argument should is torium or ./torium, skip it
             * as it has no real value to us */
            if (argv[i][0] == '-' && argv[i][1] == '-') {
                k = argv[i]+2;
                v = k;
                do v++; while (isalnum(*v) || *v == '-');
                ke = v;

                if (*v == '=') arg = true;

                *ke = '\0';
                v++;

                for (x=0; x<sz; ++x) {
                    if (strcmp(opts[x].longopt, k) == 0) {
                        found = true;
                        break;
                    }
                }

                if (!found) {
                    g_logger->errorf("unknown option `%s'", k);
                    continue;
                }

                if (opts[x].requires_argument && (!arg || *v == '\0')) {
                    g_logger->errorf("no argument given for option `%s'",
                                     opts[x].longopt);
                    continue;
                }

                id = opts[x].id;
            } else if (argv[i][0] == '-') {
                k = argv[i]+1;
                v = k;
                do v++; while (isalnum(*v) || *v == '-');

                if (*v == '=') arg = true;
                v++;

                for (x=0; x<sz; ++x) {
                    if (opts[x].shortopt == *k) {
                        found = true;
                        break;
                    }
                }

                if (!found) {
                    g_logger->errorf("unknown option `%c'", *k);
                    continue;
                }

                if (opts[x].requires_argument && (!arg || *v == '\0')) {
                    g_logger->errorf("no argument given for option `%s'",
                                     opts[x].longopt);
                    continue;
                }

                id = opts[x].id;
            } else {
                /* assume the argument is a torrent of some sort */
                v = argv[i];
                id = 3;
            }

            switch (id) {
                case 0: /* Download directory */
                {
                    bf::path p(v);

                    /* is v a valid directory? */
                    if (is_directory(p)) {
#ifdef DEBUG
                        g_logger->messagef("download_dir set to %s", v);
#endif
                        g_settings->set_option("download_dir",
                                               p.file_string().c_str());
                    } else {
                        g_logger->errorf("%s is not a directory", v);
                    }
                }
                    break;
                case 1: /* Window geometry */
                {
                    char *w, *h, *we;
                    w = v;
                    h = w;
                    do ++h; while (isdigit(*h));
                    we = h;
                    *we = '\0';
                    do ++h; while (!isdigit(*h));

                    win_width  = abs(atoi(w));
                    win_height = abs(atoi(h));
                }
                    break;
                case 2: /* Window position */
                {
                    char *x, *y, *xe;
                    x = v;
                    y = x;
                    do ++y; while (isdigit(*y));
                    xe = y;
                    *xe = '\0';
                    do ++y; while (!isdigit(*y));

                    win_x = abs(atoi(x));
                    win_y = abs(atoi(y));
                }
                    break;
                case 3: /* Add torrent */
                {
#ifdef DEBUG
                    g_logger->messagef("Adding torrent via CLI argument %s", v);
#endif
                    torrents.push_back(v);
                }
                    break;
                default:
                    /* do nothing, unknown option */
                    break;
            }
        }
    }

    window = new MainWindow(torrents, win_x, win_y, win_width, win_height);
    signal(SIGINT, sigint_handler);
    window->show();
    kit.run();

    g_logger->disable();

    delete window;
    delete g_settings;
    delete g_logger;
    delete g_torrent;

    printf("Closing connections...\n");
    return 0;
}

static void
sigint_handler(int sig)
{
    Gtk::Main::quit();
}

