#include "preferences.h"
#include "torrent.h"

Preferences::Preferences()
: lbl_download_dir("Download Directory", 0, 0),
  lbl_port_range("Port range", 0, 0),
  lbl_upload_rate_limit("Upload rate limit", 0, 0),
  lbl_download_rate_limit("Download rate limit", 0, 0),
  lbl_upload_slot_limit("Upload slot limit", 0, 0),
  lbl_refresh_rate("Refresh rate", 0, 0),
  lbl_storage_mode("Storage mode", 0, 0),
  lbl_global_uprate("Global upload rate", 0, 0),
  lbl_action_on_completion("Action on torrent completion", 0, 0),
  btn_notifications("Show notifications for completed downloads"),
  btn_upnp("Enable UPnP"),
  btn_natpmp("Enable NAT-PMP"),
  ent_download_dir_btn("Browse..."),
  box(0,0),
  m_download_dir_box(0, 0)
{
    set_title("Torium Preferences");
    set_border_width(10);

    this->property_default_width().set_value(400);

    ok_btn  = new Gtk::Button(Gtk::Stock::OK);
    cancel_btn = new Gtk::Button(Gtk::Stock::CANCEL);
    reset_btn = new Gtk::Button("Reset");
    resetdef_btn = new Gtk::Button("Reset to defaults");

    box.pack_start(tabs, 0, 0, 0);
    box.pack_start(m_btn_box, Gtk::PACK_SHRINK);

    tabs.append_page(downloads_page, "Downloads");
    tabs.append_page(network_page, "Network");
    tabs.append_page(gui_page, "GUI");

    /* i have no idea where to put them unless we put them here =O */
    tabs.append_page(extra_page, "Extra");

    ent_download_dir.property_editable() = false;
    m_download_dir_box.pack_start(ent_download_dir);
    m_download_dir_box.pack_end(ent_download_dir_btn, Gtk::PACK_SHRINK);

    downloads_page.set_homogeneous(0);
    downloads_page.pack_start(lbl_download_dir, Gtk::PACK_SHRINK);
    downloads_page.pack_start(m_download_dir_box, 0, 0);
    downloads_page.pack_start(lbl_storage_mode, Gtk::PACK_SHRINK);
    downloads_page.pack_start(ent_storage_mode, 0, 0);
    downloads_page.pack_start(btn_notifications, 0, 0);
    downloads_page.set_border_width(5);

    gui_page.pack_start(lbl_refresh_rate, Gtk::PACK_SHRINK);
    gui_page.pack_start(ent_refresh_rate, 0, 0);
    gui_page.set_border_width(5);

    extra_page.pack_start(lbl_global_uprate, Gtk::PACK_SHRINK);
    extra_page.pack_start(ent_global_uprate, 0, 0);
    extra_page.pack_start(lbl_action_on_completion, Gtk::PACK_SHRINK);
    extra_page.pack_start(ent_action_on_completion, 0, 0);
    extra_page.set_border_width(5);

    network_page.pack_start(lbl_port_range, Gtk::PACK_SHRINK);
    Gtk::HBox *portrange_box = Gtk::manage(new Gtk::HBox(true));
    portrange_box->pack_start(ent_port_start);
    portrange_box->pack_start(ent_port_end);
    network_page.pack_start(*portrange_box, false, 0);
    network_page.pack_start(lbl_upload_rate_limit, Gtk::PACK_SHRINK);
    network_page.pack_start(ent_upload_rate_limit, 0, 0);
    network_page.pack_start(lbl_download_rate_limit, Gtk::PACK_SHRINK);
    network_page.pack_start(ent_download_rate_limit, 0, 0);
    network_page.pack_start(lbl_upload_slot_limit, Gtk::PACK_SHRINK);
    network_page.pack_start(ent_upload_slot_limit, 0, 0);
    Gtk::HBox *stuff = Gtk::manage(new Gtk::HBox(true));
    stuff->pack_start(btn_upnp);
    stuff->pack_start(btn_natpmp);
    network_page.pack_start(*stuff, 0, 0);
    network_page.set_border_width(5);

    m_btn_box.pack_start(*reset_btn, Gtk::PACK_SHRINK);
    m_btn_box.pack_start(*resetdef_btn, Gtk::PACK_SHRINK);
    m_btn_box.pack_end(*cancel_btn, Gtk::PACK_SHRINK);
    m_btn_box.pack_end(*ok_btn, Gtk::PACK_SHRINK);

    ent_download_dir_btn.signal_clicked().connect(sigc::mem_fun(*this, &Preferences::on_download_dir_browse));
    cancel_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &Preferences::on_cancel_btn_clicked));
    ok_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &Preferences::on_ok_btn_clicked));
    reset_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &Preferences::reset));
    resetdef_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &Preferences::on_resetdef_btn_clicked));

    ent_port_start.signal_value_changed().connect(sigc::mem_fun(*this,
                &Preferences::on_port_start_changed));

    ent_port_end.signal_value_changed().connect(sigc::mem_fun(*this,
                &Preferences::on_port_end_changed));

    add(box);

    reset();

    /* annoying bug, we need to bind this signal AFTER we set the 
     * initial value, or else a dialog will  be shown when torium starts */
    btn_upnp.signal_toggled().connect(sigc::mem_fun(*this,
                &Preferences::restart_required_msg));
    btn_natpmp.signal_toggled().connect(sigc::mem_fun(*this,
                &Preferences::restart_required_msg));
}

/** 
 * General callback function for options requiring a restart
 * of Torium before they take effect. An example of such an 
 * option is enable_upnp.
 **/
void
Preferences::restart_required_msg()
{
    Gtk::MessageDialog msg(*this, "You need to restart Torium before this option takes effect.", false,
            Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
    msg.run();
}

/** 
 * Reset all fields in the preferences dialog to the 
 * values read from the users configuration file.
 *
 * This function must be called before the dialog is 
 * shown, since the user might have clicked "cancel"
 * before while some fields were modified.
 **/
void
Preferences::reset()
{
    tmp_port_start = atoi(g_settings->get_option("port_start"));
    tmp_port_end = atoi(g_settings->get_option("port_end"));

    ent_download_dir.set_text(g_settings->get_option("download_dir"));
    ent_download_dir.property_editable() = false;

    ent_port_end.set_range(tmp_port_start+1, 65535);
    ent_port_end.set_value(atoi(g_settings->get_option("port_end")));
    ent_port_end.set_digits(0);
    ent_port_end.set_increments(1, 10);

    ent_port_start.set_range(1, tmp_port_end-1);
    ent_port_start.set_value(tmp_port_start);
    ent_port_start.set_digits(0);
    ent_port_start.set_increments(1, 10);

    ent_upload_rate_limit.set_range(-1, 100);
    ent_upload_rate_limit.set_value(atoi(g_settings->get_option("upload_rate_limit"))/1000);
    ent_upload_rate_limit.set_digits(0);
    ent_upload_rate_limit.set_increments(5, 50);

    ent_download_rate_limit.set_range(-1, 2400);
    ent_download_rate_limit.set_value(atoi(g_settings->get_option("download_rate_limit"))/1000);
    ent_download_rate_limit.set_digits(0);
    ent_download_rate_limit.set_increments(5, 50);

    ent_upload_slot_limit.set_range(-1, 40);
    ent_upload_slot_limit.set_value(atoi(g_settings->get_option("upload_slot_limit")));
    ent_upload_slot_limit.set_digits(0);
    ent_upload_slot_limit.set_increments(5, 10);

    ent_refresh_rate.set_range(50, 5000);
    ent_refresh_rate.set_value(atoi(g_settings->get_option("refresh_rate")));
    ent_refresh_rate.set_digits(0);
    ent_refresh_rate.set_increments(50, 200);

    ent_global_uprate.set_range(-1, 200);
    if (g_torrent->global_uprate == -1)
        ent_global_uprate.set_value(0);
    else
        ent_global_uprate.set_value(g_torrent->global_uprate/1000);
    ent_global_uprate.set_digits(0);
    ent_global_uprate.set_increments(1, 5);

    ent_storage_mode.clear_items();
    ent_storage_mode.append_text("sparse");
    ent_storage_mode.append_text("allocate");
    ent_storage_mode.append_text("compact");
    ent_storage_mode.set_active_text(g_settings->get_option("storage_mode"));

    ent_action_on_completion.clear_items();
    ent_action_on_completion.append_text("nothing");
    ent_action_on_completion.append_text("remove");
    ent_action_on_completion.append_text("stop");
    ent_action_on_completion.set_active_text(g_settings->get_option("action_on_completion"));

    btn_natpmp.set_active((bool)(strcmp(g_settings->get_option("enable_natpmp"), "yes") == 0));
    btn_upnp.set_active((bool)(strcmp(g_settings->get_option("enable_upnp"), "yes") == 0));
    btn_notifications.set_active((bool)(strcmp(g_settings->get_option("notifications"), "yes") == 0));
}

void
Preferences::on_resetdef_btn_clicked()
{
    g_settings->reset_to_defaults();
    reset();
}

void
Preferences::on_port_end_changed()
{
    ent_port_start.set_range(1, ent_port_end.get_value());
}

void
Preferences::on_port_start_changed()
{
    ent_port_end.set_range(ent_port_start.get_value(), 65535);
}

/** 
 * Save the new settings to the users settings.conf
 **/
void
Preferences::saveSettings()
{
    int tmp;

    tmp = ent_download_rate_limit.get_value();
    if (tmp <= 0)
        tmp = -1;
    else
        tmp *= 1000;
    g_settings->set_option("download_rate_limit", tmp);

    tmp = ent_upload_rate_limit.get_value();
    if (tmp <= 0)
        tmp = -1;
    else
        tmp *= 1000;
    g_settings->set_option("upload_rate_limit", tmp);

    g_settings->set_option("download_dir", ent_download_dir.get_text().c_str());
    g_settings->set_option("port_start", ent_port_start.get_value());
    g_settings->set_option("port_end", ent_port_end.get_value());
    g_settings->set_option("upload_slot_limit", ent_upload_slot_limit.get_value());
    g_settings->set_option("refresh_rate", ent_refresh_rate.get_value());
    g_settings->set_option("storage_mode", ent_storage_mode.get_active_text().c_str());
    g_settings->set_option("action_on_completion", ent_action_on_completion.get_active_text().c_str());
    g_settings->set_option("notifications", btn_notifications.get_active()?"yes":"no");
    g_settings->set_option("enable_upnp", btn_upnp.get_active()?"yes":"no");
    g_settings->set_option("enable_natpmp", btn_natpmp.get_active()?"yes":"no");

    tmp = (int)ent_global_uprate.get_value();
    if (tmp > 0) {
        g_logger->messagef("Setting global upload rate limit to %d Kbit/s", tmp);
        g_torrent->set_global_uprate(tmp*1000);
    } else {
        g_torrent->set_global_uprate(-1);
    }

    g_settings->flush_config();
}

void
Preferences::on_ok_btn_clicked()
{
    saveSettings();
    hide();
}

/** 
 * Make sure the settings aren't saved if the cancel button
 * is used to terminate the dialog
 **/
void
Preferences::on_cancel_btn_clicked()
{
    hide();
}

/** 
 * Filechooser dialog for download folder
 **/
void
Preferences::on_download_dir_browse()
{
    Gtk::FileChooserDialog dialog("Choose download directory:",
            Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
    dialog.set_transient_for(*this);
    dialog.set_current_folder(g_settings->get_option("download_dir"));

    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

    Gtk::FileFilter filter_any;
    filter_any.set_name("Any files");
    filter_any.add_pattern("*");
    dialog.add_filter(filter_any);

    int result = dialog.run();

    switch(result) {
        case(Gtk::RESPONSE_OK):
        {
            ent_download_dir.set_text(dialog.get_filename().c_str());
            break;
        }
    }
}

Preferences::~Preferences()
{
    delete ok_btn;
    delete cancel_btn;
    delete reset_btn;
    delete resetdef_btn;
    /**/
}
