
#include <boost/filesystem.hpp>
#include "settings.h"
#include "properties.h"

namespace bf = boost::filesystem;

Properties::Properties(torrent_handle &th)
: h(th),
  lbl_name("Name", 0, 0),
  lbl_creator("Creator", 0, 0),
  lbl_comment("Comment", 0, 0),
  lbl_download_path("Download path", 0, 0),
  lbl_priority("Priority", 0, 0),
  btn_autorefresh("Automatically refresh list"),
  box(0,0)
{
    const torrent_info t = h.get_torrent_info();
    s = h.status();

    h.file_progress(file_progress);
    num_pieces = t.num_pieces();

    set_title("Torium "+t.name());
    set_border_width(10);

    this->property_default_width().set_value(400);

    close_btn  = new Gtk::Button(Gtk::Stock::CLOSE);

    box.pack_start(tabs, 1, 1, 1);
    box.pack_end(m_btn_box, Gtk::PACK_SHRINK);

    m_btn_box.pack_end(*close_btn, Gtk::PACK_SHRINK);

    close_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &Properties::on_close_btn_clicked));

    tabs.append_page(information_page, "Information");
    tabs.append_page(files_page, "Files");
    tabs.append_page(options_page, "Options");
    tabs.append_page(peers_page, "Peers");

    /* Information Tab */

    ent_name.set_text(t.name());
    ent_creator.set_text(t.creator());
    ent_download_path.set_text(h.save_path().string());
    Glib::RefPtr<Gtk::TextBuffer> buf_comment = ent_comment.get_buffer();
    Glib::ustring str_comment(t.comment());
    Gtk::TextBuffer::iterator i_comment = buf_comment->get_iter_at_offset(0);

    buf_comment->insert(i_comment, str_comment);
    ent_comment.set_size_request(-1, 70);

    information_page.set_homogeneous(0);
    information_page.pack_start(lbl_name, Gtk::PACK_SHRINK);
    information_page.pack_start(ent_name, 0, 0);
    information_page.pack_start(lbl_creator, Gtk::PACK_SHRINK);
    information_page.pack_start(ent_creator, 0, 0);
    information_page.pack_start(lbl_download_path, Gtk::PACK_SHRINK);
    information_page.pack_start(ent_download_path, 0, 0);
    information_page.pack_start(lbl_comment, Gtk::PACK_SHRINK);
    information_page.pack_start(ent_comment, 1, 1);
    information_page.set_border_width(5);

    /* Files Tab */

    frefTreeModel = Gtk::TreeStore::create(fcol);
    ftreeview.set_model(frefTreeModel);
    ftreeview.set_size_request(400, 250);

    /* Fill up files list */
    int index = 0;
    /* TODO: Create a proper folder structure
     *       and implement it into the treeview */
    for (torrent_info::file_iterator i = t.begin_files();
            i != t.end_files(); ++i, ++index) {
        Gtk::TreeModel::Row row = *(frefTreeModel->append());
        int first = t.map_file(index, 0, 1).piece;
        int percent;
        int priority;
        bool download = true;
        Glib::ustring priority_str;
        
        libtorrent::size_type downloaded =
            (libtorrent::size_type)(file_progress[index]*i->size);
        if (downloaded == 0)
            percent = 0;
        else if (i->size == 0)
            percent = 100;
        else
            percent = ((float)downloaded/i->size)*100;

        priority = h.piece_priority(first);
        if (percent == 100) {
            priority_str = "Done";
            download = false;
        } else {
            switch (priority) {
                case 0:
                    priority_str = "Skip";
                    download = false;
                    break;
                case 1:
                    priority_str = "Normal";
                    break;
                case 4:
                    priority_str = "High";
                    break;
                case 6:
                    priority_str = "Max";
                    break;
                default:
                    priority_str = "??";
                    break;
            }
        }

        row[fcol.index]        = index;
        row[fcol.size]         = i->size;
        row[fcol.name]         = i->path.string();
        row[fcol.percent]      = percent;
        row[fcol.download]     = download;
        row[fcol.priority]     = priority;
        row[fcol.priority_str] = priority_str;
    }

    Gtk::TreeView::Column *col_file = 
        Gtk::manage(new Gtk::TreeView::Column("File"));
    Gtk::TreeView::Column *col_download = 
        Gtk::manage(new Gtk::TreeView::Column("Download"));
    Gtk::TreeView::Column *col_progress = 
        Gtk::manage(new Gtk::TreeView::Column("Progress"));
    Gtk::TreeView::Column *col_priority = 
        Gtk::manage(new Gtk::TreeView::Column("Priority"));

    Gtk::CellRendererText *cell_file = 
        Gtk::manage(new Gtk::CellRendererText);
    Gtk::CellRendererProgress *cell_progress =
        Gtk::manage(new Gtk::CellRendererProgress);
    Gtk::CellRendererToggle *cell_download =
        Gtk::manage(new Gtk::CellRendererToggle);
    Gtk::CellRendererText *cell_priority = 
        Gtk::manage(new Gtk::CellRendererText);

    col_file->pack_start(*cell_file);
    col_progress->pack_start(*cell_progress);
    col_download->pack_start(*cell_download);
    col_priority->pack_start(*cell_priority);

    ftreeview.append_column(*col_file);
    ftreeview.append_column(*col_progress);
    ftreeview.append_column(*col_download);
    ftreeview.append_column(*col_priority);

    col_file->set_resizable();
    col_file->set_expand();
    col_progress->set_fixed_width(70);
    col_progress->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
    col_download->set_fixed_width(70);
    col_download->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
    col_priority->set_fixed_width(60);
    col_priority->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);

    col_file->add_attribute(cell_file->property_text(), fcol.name);
    col_progress->add_attribute(cell_progress->property_value(), fcol.percent);
    col_download->add_attribute(cell_download->property_active(), fcol.download);
    col_priority->add_attribute(cell_priority->property_text(), fcol.priority_str);

    cell_download->signal_toggled().connect(sigc::mem_fun(*this,
                &Properties::on_download_toggled));

    f_scrolledwindow.add(ftreeview);
    f_scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    files_page.pack_start(f_scrolledwindow);

    /* Options Tab */

    options_page.pack_start(lbl_priority, Gtk::PACK_SHRINK);
    options_page.pack_start(ent_priority, 0, 0);

    ent_priority.clear_items();
    ent_priority.append_text("skip");
    ent_priority.append_text("normal");
    ent_priority.append_text("high");
    ent_priority.append_text("max");
    int priority = h.piece_priority(1);
    ent_priority.set_active_text(prio_name(priority));

    ent_priority.signal_changed().connect(sigc::mem_fun(*this,
                &Properties::on_prio_changed));

    /* Peers Tab */

    prefTreeModel = Gtk::ListStore::create(pcol);
    ptreeview.set_model(prefTreeModel);
    ptreeview.set_size_request(400, 250);

    /* Fill up peer list */
    refresh_peerlist();

    Gtk::TreeView::Column *col_cc = 
        Gtk::manage(new Gtk::TreeView::Column("CC"));
    Gtk::TreeView::Column *col_ip = 
        Gtk::manage(new Gtk::TreeView::Column("IP"));
    Gtk::TreeView::Column *col_client = 
        Gtk::manage(new Gtk::TreeView::Column("Client"));
    Gtk::TreeView::Column *col_dspeed = 
        Gtk::manage(new Gtk::TreeView::Column("Down"));
    Gtk::TreeView::Column *col_uspeed = 
        Gtk::manage(new Gtk::TreeView::Column("Up"));

    Gtk::CellRendererPixbuf *cell_cc = 
        Gtk::manage(new Gtk::CellRendererPixbuf);
    Gtk::CellRendererText *cell_ip = 
        Gtk::manage(new Gtk::CellRendererText);
    Gtk::CellRendererText *cell_client = 
        Gtk::manage(new Gtk::CellRendererText);
    Gtk::CellRendererText *cell_dspeed = 
        Gtk::manage(new Gtk::CellRendererText);
    Gtk::CellRendererText *cell_uspeed = 
        Gtk::manage(new Gtk::CellRendererText);

    col_cc->pack_start(*cell_cc);
    col_ip->pack_start(*cell_ip);
    col_client->pack_start(*cell_client);
    col_dspeed->pack_start(*cell_dspeed);
    col_uspeed->pack_start(*cell_uspeed);

    ptreeview.append_column(*col_cc);
    ptreeview.append_column(*col_ip);
    ptreeview.append_column(*col_client);
    ptreeview.append_column(*col_dspeed);
    ptreeview.append_column(*col_uspeed);

    col_cc->set_sort_column(pcol.cc);
    col_ip->set_sort_column(pcol.ipaddr);
    col_client->set_sort_column(pcol.client);
    col_dspeed->set_sort_column(pcol.dspeed);
    col_uspeed->set_sort_column(pcol.uspeed);

    col_cc->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_ip->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_client->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_dspeed->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_uspeed->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);

    col_cc->add_attribute(cell_cc->property_pixbuf(), pcol.cc_img);
    col_ip->add_attribute(cell_ip->property_text(), pcol.ipaddr);
    col_client->add_attribute(cell_client->property_text(), pcol.client);
    col_dspeed->add_attribute(cell_dspeed->property_text(), pcol.dspeed_str);
    col_uspeed->add_attribute(cell_uspeed->property_text(), pcol.uspeed_str);

    refreshpeers_btn  = new Gtk::Button(Gtk::Stock::REFRESH);
    p_btn_box.pack_end(btn_autorefresh, Gtk::PACK_SHRINK);
    p_btn_box.pack_end(*refreshpeers_btn, Gtk::PACK_SHRINK);

    refreshpeers_btn->signal_clicked().connect(sigc::mem_fun(*this,
                &Properties::on_refreshpeers_btn_clicked));

    p_scrolledwindow.add(ptreeview);
    peers_page.pack_end(p_btn_box, Gtk::PACK_SHRINK);
    p_scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    peers_page.pack_start(p_scrolledwindow);

    add(box);
    show_all();

    h.get_peer_info(peers);
    refresh_peerlist();

    btn_autorefresh.set_active(true);
    Glib::signal_timeout().connect(sigc::mem_fun(*this, &Properties::on_timer), 1500);
}

Properties::~Properties()
{
    delete refreshpeers_btn;
    delete close_btn;

    hide();
}

void
Properties::on_close_btn_clicked()
{
    hide();
}

/** 
 * Change a files priority in the files tab
 * TODO: We should probably use a vector here instead of looping through every
 *       piece
 **/
void
Properties::change_file_priority(Gtk::TreeRow row, int prio)
{
    const torrent_info t = h.get_torrent_info();
    int index;
    libtorrent::size_type size;
    int first;
    int last;
    int n;
    Glib::ustring prio_str;

    index = row[fcol.index];
    size  = row[fcol.size];
    first = t.map_file(index, 0, 1).piece;
    last  = t.map_file(index, size-1, 1).piece;

    if (first == last) {
        h.piece_priority(first, prio);
    } else {
        for (n=first; n!=last; n++) {
            h.piece_priority(n, prio);
        }
    }
    if (row[fcol.percent] == 100) {
        prio_str = "Done";
    } else {
        prio_str = prio_name(prio);
        row[fcol.priority] = prio;
    }
    row[fcol.priority_str] = prio_str;
}

void
Properties::change_torrent_priority(int prio)
{
    std::vector<int> pieces;
    pieces.assign(num_pieces, prio);
    h.prioritize_pieces(pieces);
}


/** 
 * One of the download checkboxes in the files tab has been toggled
 **/
void
Properties::on_download_toggled(Glib::ustring path)
{
    Gtk::TreeRow row = *(frefTreeModel->get_iter(path));
    bool download = row[fcol.download];

    download = !download;
    row[fcol.download] = download;

    if (!download) {
        change_file_priority(row, 0);
    } else {
        change_file_priority(row, 1);
    }
}

/** 
 * The torrent priority combobox has been changed under the options tab
 **/
void
Properties::on_prio_changed()
{
    std::string new_prio_str = ent_priority.get_active_text();
    int new_prio = 1;

    if (new_prio_str == "skip")
        new_prio = 0;
    else if (new_prio_str == "high")
        new_prio = 4;
    else if (new_prio_str == "max")
        new_prio = 6;

    change_torrent_priority(new_prio);
}

void
Properties::on_refreshpeers_btn_clicked()
{
    refresh_peerlist();
}

/** 
 * Clear the peerlist
 **/
void
Properties::clear_peerlist()
{
    prefTreeModel->clear();
}

/** 
 * Refreshes the peerlist by first clearing it then reentering all peers
 * TODO: We need a better solution for this, but I got sick of trying to get
 *       rid of the nasty segmentation fault that occured when trying to loop
 *       through them one by one.
 **/
void
Properties::refresh_peerlist()
{
    clear_peerlist();
    peers_sz = peers.size();
    int i;
    for (i = 0; i < peers_sz; ++i) {
        Gtk::TreeModel::Row row = *(prefTreeModel->append());
        char c[512];

        if (strlen(peers[i].country) && peers[i].country[0] != '!') {
            peers[i].country[0] = tolower(peers[i].country[0]);
            peers[i].country[1] = tolower(peers[i].country[1]);
            snprintf(c, 512, "%s%cflags%c%s.png", g_settings->share_path,
                    DIR_SEP, DIR_SEP, peers[i].country);
        } else {
            snprintf(c, 512, "%s%cflags%c00.png", g_settings->share_path,
                    DIR_SEP, DIR_SEP);
        }

        bf::path p(c);
        if (bf::exists(p)) {
            Glib::RefPtr<Gdk::Pixbuf> image = Gdk::Pixbuf::create_from_file(p.string());
            row[pcol.cc_img] = image;
        }

        row[pcol.cc]         = peers[i].country;
        row[pcol.ipaddr]     = peers[i].ip.address().to_string();
        row[pcol.client]     = Glib::locale_to_utf8(peers[i].client);
        row[pcol.dspeed]     = peers[i].down_speed;
        row[pcol.uspeed]     = peers[i].up_speed;
        row[pcol.dspeed_str] = fix_rate(peers[i].down_speed);
        row[pcol.uspeed_str] = fix_rate(peers[i].up_speed);
    }
}

/** 
 * Occurs once every 1.5 seconds
 * Refreshes the peerlist if autorefresh is activated, otherwise just update
 * the peerlist
 **/
bool
Properties::on_timer()
{
    h.get_peer_info(peers);
    if (btn_autorefresh.get_active())
        refresh_peerlist();
    return true;
}
