#ifndef __PROPERTIES_H_
#define __PROPERTIES_H_

#include <gtkmm.h>
#include <libtorrent/torrent_handle.hpp>

#include "tools.h"

using namespace libtorrent;

class Properties : public Gtk::Window
{
  public:
	Properties(torrent_handle &th);
	virtual ~Properties();
    torrent_handle &h;
    torrent_status s;

    std::vector<float> file_progress;
    std::vector<peer_info> peers;
    void change_file_priority(Gtk::TreeRow row, int prio);
    void change_torrent_priority(int prio);
  
  protected:
    void save();
    void on_close_btn_clicked();
    void on_refreshpeers_btn_clicked();
    void on_download_toggled(Glib::ustring path);
    void on_prio_changed();
    bool on_timer();
    void clear_peerlist();
    void refresh_peerlist();

    int num_pieces;
    int peers_sz;

    Gtk::Label lbl_name;
    Gtk::Label lbl_creator;
    Gtk::Label lbl_comment;
    Gtk::Label lbl_download_path;
    Gtk::Label lbl_priority;

    Gtk::Entry ent_name;
    Gtk::Entry ent_creator;
    Gtk::Entry ent_download_path;
    Gtk::TextView ent_comment;
    Gtk::ComboBoxText ent_priority;
    Gtk::CheckButton btn_autorefresh;
    Gtk::Button *refreshpeers_btn;
    Gtk::Button *close_btn;

    class FileColumns : public Gtk::TreeModel::ColumnRecord
    {
      public:
        FileColumns() {
            add(index);
            add(size);
            add(name);
            add(percent);
            add(download);
            add(priority);
            add(priority_str);
            add(activatable);
        }

        Gtk::TreeModelColumn<int> index;
        Gtk::TreeModelColumn<libtorrent::size_type> size;
        Gtk::TreeModelColumn<Glib::ustring> name;
        Gtk::TreeModelColumn<int> percent;
        Gtk::TreeModelColumn<bool> download;
        Gtk::TreeModelColumn<int> priority;
        Gtk::TreeModelColumn<Glib::ustring> priority_str;
        Gtk::TreeModelColumn<bool> activatable;
    };

    class PeerColumns : public Gtk::TreeModel::ColumnRecord
    {
      public:
        PeerColumns() {
            add(index);
            add(cc);
            add(cc_img);
            add(ipaddr);
            add(client);
            add(dspeed);
            add(uspeed);
            add(dspeed_str);
            add(uspeed_str);
        }

        Gtk::TreeModelColumn<int> index;
        Gtk::TreeModelColumn<Glib::ustring> cc;
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > cc_img;
        Gtk::TreeModelColumn<Glib::ustring> ipaddr;
        Gtk::TreeModelColumn<Glib::ustring> client;
        Gtk::TreeModelColumn<float> dspeed;
        Gtk::TreeModelColumn<float> uspeed;
        Gtk::TreeModelColumn<Glib::ustring> dspeed_str;
        Gtk::TreeModelColumn<Glib::ustring> uspeed_str;
    };

    FileColumns fcol;
    Gtk::TreeView ftreeview;
    Glib::RefPtr<Gtk::TreeStore> frefTreeModel;
    Gtk::ScrolledWindow f_scrolledwindow;

    PeerColumns pcol;
    Gtk::TreeView ptreeview;
    Glib::RefPtr<Gtk::ListStore> prefTreeModel;
    Gtk::ScrolledWindow p_scrolledwindow;

    Gtk::Notebook tabs;
    Gtk::VBox information_page;
    Gtk::VBox files_page;
    Gtk::VBox options_page;
    Gtk::VBox peers_page;
    Gtk::VBox box;

    Gtk::HBox m_btn_box;
    Gtk::HBox p_btn_box;
};
#endif
