#ifndef _SETTINGS__H_
#define _SETTINGS__H_

#include <string>
#include <fstream>
#include <map>

/** 
 * Global settings loader and handler.
 *
 * This class loads the users settings file, usuaally
 * ~/.torium/settings.conf and creates a mapping for 
 * each options value.
 *
 * The functions set_option() are used to set an option,
 * options are always stored as char pointers internally,
 * and get_option() will return the option as a char*. It
 * is up to the callee to convert the option value to the 
 * desired type.
 *
 * Note that comments are allowed in the configuration 
 * file, but when the class writes the configuration file
 * it will not remember comments and thus they will be 
 * discarded.
 **/
class Settings
{
  public:
    Settings();
    ~Settings();
    bool init();
    bool flush_config();
    const char * get_option(const char *option);
    bool set_option(const char *option, const char *value);
    bool set_option(const char *option, const int value);
    bool set_option(const char *option, const std::string value);
    void reset_to_defaults();
    bool load_config(const char *path);
    std::map<std::string, std::string> opt_values;

    std::string home_path;
    char *share_path;
    std::string conf_dir;
    std::string torrent_dir;
    std::string fastresume_dir;
    std::string monitor_dir;
    std::string download_dir;
    std::string states_dir;
    std::string pid_file;
    std::string pipe_file;

  private:
    std::string conf_file;
    std::fstream cfg_h;
};

/* global object, available to all files that include this 
 * header file. Used to set and get option values.
 *
 * declared in settings.cc */
extern Settings *g_settings;

#endif
