#ifndef _TORRENT__H_
#define _TORRENT__H_

#include "settings.h"
#include "logger.h"
#include "config.h"
#include "tools.h"

#include <string>
#include <boost/filesystem.hpp>
#include <boost/format.hpp>
#include <exception>

#include "libtorrent/torrent_handle.hpp"
#include "libtorrent/alert.hpp"
#include "libtorrent/ip_filter.hpp"

using namespace libtorrent;
namespace bf = boost::filesystem;
typedef std::multimap<std::string, torrent_handle> handles_t;

/**
 * Class used for each added torrent
 **/
class Torrent
{
  public:
    Torrent();
    ~Torrent();

    handles_t handles;
    bf::path fastresume_dir;
    int global_uprate;

    struct insufficent_files: std::exception
    {
        virtual const char* what() const throw() { return "insufficent files"; }
    };

    struct invalid_path: std::exception
    {
        virtual const char* what() const throw() { return "invalid_path"; }
    };

    struct insufficent_trackers: std::exception
    {
        virtual const char* what() const throw() { return "insufficent trackers"; }
    };

    const char* add_torrent(const std::string path, const std::string download_path);
    bool remove_torrent(const std::string name);
    bool get_torrent(const std::string name, torrent_handle &h);
    void get_num_torrents(int *torrents, int *completed_torrents);
    unsigned get_total_eta();
    const size_type get_total_downloaded();
    const size_type get_total_uploaded();
    const float download_rate();
    const float upload_rate();
    void update_rates();
    std::auto_ptr<alert> pop_alert();
    bool set_ip_filter(ip_filter const &filter);
    bool set_torrent_priority(int new_priority);
    bool set_file_priority(const std::string name, std::vector<int> pieces);
    bool resume(torrent_handle *h);
    bool pause(torrent_handle *h);
    int toggle_pause(const std::string name);
    void force_recheck(const std::string name);
    void force_reannounce(const std::string name);
    void set_global_uprate(int uprate);
    void add_files(torrent_info &t, bf::path const &p, bf::path const &l);
    void create_torrent(std::vector<Glib::ustring> files, std::vector<Glib::ustring> trackers, const char *path);
    bool has_downloaded();

    void save_fastresume();

    std::vector<torrent_handle> get_torrents();

  protected:
    session *ses;
};

extern Torrent *g_torrent;

#endif
