#include "treeview.h"
#ifndef WIN32
#include <libnotify/notify.h>
#endif
#include <boost/lexical_cast.hpp>

TreeView::TreeView()
{
    m_refTreeModel = Gtk::ListStore::create(col);
    set_model(m_refTreeModel);

    force_update = false;
    update_tooltip = false;

    property_show_expanders() = false;

    Gtk::TreeView::Column *col_htorrent = 
        Gtk::manage(new Gtk::TreeView::Column("Torrent"));
    Gtk::TreeView::Column *col_hdspeed = 
        Gtk::manage(new Gtk::TreeView::Column("Down"));
    Gtk::TreeView::Column *col_huspeed = 
        Gtk::manage(new Gtk::TreeView::Column("Up"));
    Gtk::TreeView::Column *col_hstatus = 
        Gtk::manage(new Gtk::TreeView::Column("Status"));
    Gtk::TreeView::Column *col_heta = 
        Gtk::manage(new Gtk::TreeView::Column("ETA"));
    Gtk::TreeView::Column *col_hprogress = 
        Gtk::manage(new Gtk::TreeView::Column("Progress"));

    Gtk::CellRendererPixbuf *cellPixbuf = 
        Gtk::manage(new Gtk::CellRendererPixbuf);

    Gtk::CellRendererText *cellText = 
        Gtk::manage(new Gtk::CellRendererText);

    Gtk::CellRendererText *dspeed_text = 
        Gtk::manage(new Gtk::CellRendererText);

    Gtk::CellRendererText *uspeed_text = 
        Gtk::manage(new Gtk::CellRendererText);

    Gtk::CellRendererText *status_text = 
        Gtk::manage(new Gtk::CellRendererText);

    Gtk::CellRendererText *eta_text = 
        Gtk::manage(new Gtk::CellRendererText);

    Gtk::CellRendererProgress *progressBar = 
        Gtk::manage(new Gtk::CellRendererProgress);

    col_htorrent->pack_start(*cellPixbuf);
    col_htorrent->pack_start(*cellText);
    col_hdspeed->pack_start(*dspeed_text);
    col_huspeed->pack_start(*uspeed_text);
    col_hstatus->pack_start(*status_text);
    col_heta->pack_start(*eta_text);
    col_hprogress->pack_start(*progressBar);

    append_column(*col_htorrent);
    append_column(*col_hdspeed);
    append_column(*col_huspeed);
    append_column(*col_hstatus);
    append_column(*col_heta);
    append_column(*col_hprogress);

    col_htorrent->set_sort_column(col.str_name);
    col_hdspeed->set_sort_column(col.dspeed);
    col_huspeed->set_sort_column(col.uspeed);
    col_hstatus->set_sort_column(col.status_num);
    col_heta->set_sort_column(col.eta);
    col_hprogress->set_sort_column(col.percentage);

    col_htorrent->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_hdspeed->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_huspeed->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_hstatus->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
    col_heta->set_sizing(Gtk::TREE_VIEW_COLUMN_GROW_ONLY);
    col_hprogress->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);

    col_htorrent->add_attribute(cellPixbuf->property_stock_id(), col.stockid);
    col_htorrent->add_attribute(cellText->property_markup(), col.str_name);
    col_hdspeed->add_attribute(dspeed_text->property_text(), col.dspeed_str);
    col_huspeed->add_attribute(uspeed_text->property_text(), col.uspeed_str);
    col_hstatus->add_attribute(status_text->property_text(), col.status);
    col_heta->add_attribute(eta_text->property_text(), col.eta);
    col_hprogress->add_attribute(progressBar->property_value(), col.percentage);

    selection = get_selection();
    selection->signal_changed().connect(sigc::mem_fun(*this,
                &TreeView::on_selection_changed));

    /* Double clicking or pressing enter will "activate"
     * the currently selected torrent row and expand/collapse it */
    signal_row_activated().connect(sigc::mem_fun(*this,
                &TreeView::on_treeview_row_activated));

    /* drag'n'drop */
    std::list<Gtk::TargetEntry> targetlist;
    targetlist.push_back(Gtk::TargetEntry("application/x-bittorrent"));
    targetlist.push_back(Gtk::TargetEntry("STRING"));
    drag_dest_set(targetlist);
    signal_drag_data_received().connect(
            sigc::mem_fun(*this, &TreeView::on_data_receive));

    Gtk::Menu::MenuList &ml_popup_menu = popup_menu.items();

    ml_popup_menu.push_back(Gtk::Menu_Helpers::MenuElem(
        "_Properties",
        sigc::mem_fun(*this, &TreeView::on_popup_properties)));

    ml_popup_menu.push_back(Gtk::Menu_Helpers::MenuElem(
        "Pause/Resume",
        sigc::mem_fun(*this, &TreeView::on_popup_pauseresume_torrent)));

    ml_popup_menu.push_back(Gtk::Menu_Helpers::MenuElem(
        "_Force reannounce",
        sigc::mem_fun(*this, &TreeView::on_popup_reannounce_torrent)));

    ml_popup_menu.push_back(Gtk::Menu_Helpers::MenuElem(
        "_Go to download directory",
        sigc::mem_fun(*this, &TreeView::on_popup_goto_download_dir)));
    
    ml_popup_menu.push_back(Gtk::Menu_Helpers::MenuElem(
        "_Remove torrent",
        sigc::mem_fun(*this, &TreeView::on_popup_remove_torrent)));

    popup_menu.accelerate(*this);

    /* Initialize libnotify */
#ifndef WIN32
    notify_init("Torium");
#endif

    if(tolower(g_settings->get_option("ip_filter")[0]) == 'y') {
        load_ip_filters();
    }
}

void
TreeView::expand_all()
{
    typedef Gtk::TreeModel::Children type_children;
    type_children children = m_refTreeModel->children();

    for (type_children::iterator iter = children.begin();
            iter != children.end(); ++iter) {
        (*iter)[col.expanded] = 1;
    }
    refreshTorrents();
}

void
TreeView::collapse_all()
{
    typedef Gtk::TreeModel::Children type_children;
    type_children children = m_refTreeModel->children();

    for (type_children::iterator iter = children.begin();
            iter != children.end(); ++iter) {
        (*iter)[col.expanded] = 0;
    }
    refreshTorrents();
}

void
TreeView::on_data_receive(
        const Glib::RefPtr<Gdk::DragContext> &context, int x, int y,
        const Gtk::SelectionData& selection_data, guint info, guint time)
{
    const guint len = selection_data.get_length();
    if(len >= 0 && selection_data.get_format() == 8) {
        std::string s(selection_data.get_text());
        std::vector<const char*> uris;
        s = urldecode(s);
        size_t len = s.size();
        unsigned int i;
        /* recursively check for torrent files in the given drag'n'drop string */
        for(i = 0; i < len; i++) {
            if(s.substr(0,7).compare("file://") == 0) {
                s = s.replace(0, 7, "");
                s.resize(len-7);
                len = s.size();
            } else if(s[i] == '\n') {
                if(s.substr(i-7, 6) == "torium") {
                    g_logger->error(
" _______________________\n"
"< Have you mooed today? >\n"
" -----------------------\n"
"       \\   ^__^\n"
"        \\  (oo)\\_______\n"
"           (__)\\       )\\/\\\n"
"                ||----w |\n"
"                ||     ||");
                    g_logger->message("");
                } else if(s.substr(i-9, 8) == ".torrent") {
                    uris.push_back(s.substr(0, i-1).c_str());
                }
                s = s.substr(i+1);
                len = s.size();
                i = 0;
            }
        }
        unsigned int sz = uris.size();
        for(i = 0; i<sz; i++) {
            if(add_torrent(uris[i])) {
                g_logger->messagef("Added %s", uris[i]);
            } else {
                g_logger->errorf("Unable to add %s", uris[i]);
            }
        }
    }
    context->drag_finish(false, false, time);
}

/**
 * Called by libnotify when the user clicks the "open download folder"
 * button
 **/
#ifndef WIN32
static void
notify_open_folder(NotifyNotification *n, const char *action,
                   char *path)
{
    char *argv[] = {"xdg-open", path, 0};
    g_spawn_async(0, argv, 0, G_SPAWN_SEARCH_PATH, 0, 0, 0, 0);
}

#endif

/** 
 * Handle a completed download; notify the user and mark it as completed.
 **/
void
TreeView::handle_completed(torrent_handle *h)
{
    if(!h->is_valid() || !h->has_metadata())
        return;

    try {
        torrent_status s = h->status();
        const char *nm = h->get_torrent_info().name().c_str();
        int n=0, t=0;

        g_torrent->get_num_torrents(&n, &t);

        /** 
         * torrents that are completed before startup should not be displayed as a popup.
         * fastresume will be written in the destructor
         **/
        bf::path fastresume_dir(g_settings->fastresume_dir);
        entry data = h->write_resume_data();

        if (!bf::is_directory(fastresume_dir)) {
            g_logger->errorf("%s is not a directory",
                    g_settings->fastresume_dir.c_str());
        } else {
            const torrent_info t = h->get_torrent_info();
            char c[512];
            int r;
            r = sprintf(c, "%s%c%s.fastresume", g_settings->fastresume_dir.c_str(),
                        DIR_SEP, t.name().c_str());
            if (r <= 0) {
                fprintf(stderr, "error\n");
                abort();
            }

            bf::path fastresume_path(c);

            if(bf::exists(fastresume_path)) {
                bf::ofstream out(fastresume_path, std::ios_base::binary);
                out.unsetf(std::ios_base::skipws);
                bencode(std::ostream_iterator<char>(out), data);
            }
        }

        if (g_torrent->has_downloaded()) {
#ifndef WIN32
            if (g_settings->get_option("notifications")[0] == 'y') {
                /** 
                 * Send a notification to the user
                 **/
                NotifyNotification *n = notify_notification_new("Torrent Complete", nm, "Torium"
#if !defined(NOTIFY_VERSION_MINOR) || (NOTIFY_VERSION_MAJOR == 0 && NOTIFY_VERSION_MINOR < 7)
                                            , 0
#endif
                                            );
                notify_notification_add_action(n, "folder", "Open download folder", 
                                                NOTIFY_ACTION_CALLBACK(notify_open_folder), 
                                                (void*)h->save_path().string().c_str(), 0);
                notify_notification_show(n, 0);
            }
#endif

            g_logger->messagef("Torrent completed, Number of torrents: %d  Completed torrents: %d",
                               n, t);

            /* Ugly solution, will be nicer when there's one class for one Torrent
             * and one class for the all the torrents */

            typedef Gtk::TreeModel::Children type_children;
            type_children children = m_refTreeModel->children();
            for (type_children::iterator iter = children.begin();
                    iter != children.end(); ++iter) {
                if ((*iter)[col.name] == h->name()) {
                    int action_on_completion = boost::lexical_cast<int>((*iter)[col.action_on_completion]);
                    if ((*iter)[col.action_on_completion] == 1) {
                        /* Remove torrent on completion */
                        g_logger->messagef("Removing torrent %s...", h->name().c_str());

                        std::string tmp_name(boost::lexical_cast<std::string>((*iter)[col.name]));
                        remove_torrent(tmp_name, true);
                    } else if ((*iter)[col.action_on_completion] == 2) {
                        /* Stop torrent on completion */
                        g_logger->messagef("Pausing torrent %s...",
                                h->name().c_str());
                        g_torrent->pause(h);
                    } else {
                        g_logger->messagef("action_on_completion = %d",
                                action_on_completion);
                    }
                    break;
                }
            }
        }
    } catch (std::exception &ex) {
        g_logger->errorf("treeview::handle_completed(): %s", ex.what());
    }
}

/** 
 * Handle all torrent alerts.
 *
 * This functions will check for alerts sent by libtorrent. An alert may 
 * for example be a completed download or a message. Completed downloads
 * will be sent to handle_completed().
 **/
void
TreeView::check_torrent_alerts()
{
    std::auto_ptr<alert> pa;
    for (pa = g_torrent->pop_alert();
         pa.get(); pa = g_torrent->pop_alert()) {
        char msg[512];

        if (torrent_finished_alert* ptr = dynamic_cast<torrent_finished_alert*>(pa.get())) {
            torrent_handle &h = ptr->handle;
            if (!h.is_valid()) {
                continue;
            }
            handle_completed(&h);

            /* XXX: handle_completed return 1 if a torrent has been removed,
             * otherwise 0
             *
             * In case the torrent has been removed, we have to check
             * the handle validity again */
            if (!h.is_valid()) {
                continue;
            }
            update_tooltip = true;

            const char *nm = h.get_torrent_info().name().c_str();
            if (snprintf(msg, 512, "Finished downloading: %.487s", nm) <= 0)
                break;
        } else {
#if LIBTORRENT_VERSION_MINOR >= 14
            const char *alert_msg = pa->message().c_str();
#else
            const char *alert_msg = pa->msg().c_str();
#endif
            if (snprintf(msg, 512, "misc: %.500s", alert_msg) <= 0)
                break;
        }

        if (pa->severity() == alert::fatal)
            g_logger->error(msg);
        else if (pa->severity() == alert::warning)
            g_logger->warning(msg);
        else
            g_logger->message(msg);
    }
}

void 
TreeView::refreshTorrents()
{
    typedef Gtk::TreeModel::Children type_children;
    type_children children = m_refTreeModel->children();
    Gtk::TreeModel::Row torrent;
    for (handles_t::iterator i = g_torrent->handles.begin();
            i != g_torrent->handles.end();) {
        torrent_handle& h = i->second;
        if(!h.is_valid()) {
            g_torrent->handles.erase(i++);
            continue;
        } else {
            ++i;
        }

        bool success = false;
        for(type_children::iterator iter = children.begin();
                iter != children.end(); ++iter) {
            if((*iter)[col.name] == h.name()) {
                torrent = *iter;
                success = true;
                break;
            }
        }

        if (!success) continue;

        torrent_status s = h.status();
        int percentage = (int)(s.progress*100);
        torrent[col.percentage]     = percentage;
        torrent[col.num_seeders]    = s.num_seeds;
        torrent[col.num_peers]      = s.num_peers;
        torrent[col.downloaded]     = (long)s.total_done;
        torrent[col.dspeed]         = s.download_rate;
        torrent[col.uspeed]         = s.upload_rate;

        int state;

        if (h.is_paused()) {
            state = STATE_PAUSED;
            torrent[col.stockid] = Gtk::Stock::MEDIA_PAUSE;
        } else {
            state = s.state;
            
            if (state == STATE_SEEDING) {
                torrent[col.stockid] = Gtk::Stock::GO_UP;
            } else if(s.download_rate > 1000) {
                torrent[col.stockid] = Gtk::Stock::GO_DOWN;
            } else {
                torrent[col.stockid] = Gtk::Stock::NO;
            }
        }

        std::string s_dspeed = "-";
        std::string s_uspeed = "-";
        std::string s_eta    = "-";

        switch (state) {
            case STATE_DOWNLOADING:
            case STATE_DOWNLOADING_2:
            default:
                s_dspeed = fix_rate(s.download_rate);
                s_eta    = fix_time(get_eta(s.download_rate,
                                    torrent[col.total_size], s.total_done));
            case STATE_SEEDING:
            case STATE_FINISHED:
                s_uspeed = fix_rate(s.upload_rate);
            case STATE_PAUSED:
                break;
        }
        
        torrent[col.dspeed_str]     = s_dspeed;
        torrent[col.uspeed_str]     = s_uspeed;
        torrent[col.eta]            = s_eta;
        torrent[col.status_num]     = state;

        torrent[col.status] = state_name(state);
        update_torrent(torrent);
    }
}

/**
 * Replace the normal mouse button press event if necessary
 **/
bool
TreeView::on_button_press_event(GdkEventButton* event)
{
    bool retval = Gtk::TreeView::on_button_press_event(event);
    if((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
        Glib::RefPtr<Gtk::TreeView::Selection> refSelection = get_selection();
        if(refSelection) {
            Gtk::TreeModel::iterator iter = refSelection->get_selected();
            if(iter) {
                Gtk::TreeModel::Row torrent = *iter;
                /* TODO: refresh the submenu so that the proper
                 * radio button is active */

                popup_menu.popup(event->button, event->time);
            }
        }
        return true;
    }

    return retval;
}

void
TreeView::on_popup_properties()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = get_selection();
    if(refSelection) {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter) {
            std::string name(boost::lexical_cast<std::string>((*iter)[col.name]));

            torrent_handle h;
            if (g_torrent->get_torrent(name, h)) {
                Properties window(h);
                Gtk::Main::run(window);
            } else {
                g_logger->error("unable to init properties window");
            }
        }
    }
}

/** 
 * load ip filters from CONF_DIR/ipfilter
 *
 * TODO: Read further flags,
 * eventually the 'comments' from an ip-filter file as well
 **/
void
TreeView::load_ip_filters() {
    libtorrent::ip_filter filter;
    char filename[512];
    snprintf(filename, 512, "%s%cipfilter",
            g_settings->conf_dir.c_str(), DIR_SEP);
    std::ifstream in(filename);
    if (in.is_open()) {
#ifdef DEBUG
        clock_t start = clock();
#endif
        char *p;
        int a=0, b=0, c=0, d=0;
        char ln[64];
        while (!in.eof()) {
            in.getline(ln, 64);
            p = ln;

            if (*p == '\0')
                break;

            /* filter out comments and empty lines */
            if (*p == '#' || *p == '\r' || *p == '\n') {
                continue;
            }

            /* Get the first IP address */
            a = atoi(p);
            do ++p; while (isdigit(*p));
            b = atoi(++p);
            do ++p; while (isdigit(*p));
            c = atoi(++p);
            do ++p; while (isdigit(*p));
            d = atoi(++p);
            do ++p; while (isdigit(*p));
            address_v4 ip_start((a << 24) + (b << 16) + (c << 8) + d);

            /* Skip any nonsense between the two IP addresses */
            do ++p; while (!isdigit(*p));

            /* Get the second IP address */
            a = atoi(p);
            do ++p; while (isdigit(*p));
            b = atoi(++p);
            do ++p; while (isdigit(*p));
            c = atoi(++p);
            do ++p; while (isdigit(*p));
            d = atoi(++p);
            do ++p; while (isdigit(*p));
            address_v4 ip_last((a << 24) + (b << 16) + (c << 8) + d);

            /* Add the two IP addresses as a range to be ip filter to be blocked */
            filter.add_rule(ip_start, ip_last, ip_filter::blocked);
        }
#ifdef DEBUG
        g_logger->messagef("Completed loading IP filter in %.2fs",
                           ((double)(clock()-start))/CLOCKS_PER_SEC);
#endif
    }
    g_torrent->set_ip_filter(filter);
}

void
TreeView::remove_torrent(const std::string name, bool row)
{
    g_logger->messagef("Attempting to remove torrent %s", name.c_str());

    typedef Gtk::TreeModel::Children type_children;
    type_children children = m_refTreeModel->children();
    if (row) {
        for (type_children::iterator iter = children.begin();
                iter != children.end(); ++iter) {
            if ((*iter)[col.name] == name) {
                bf::path p((*iter)[col.path]);
                if(bf::exists(p)) {
                    bf::remove(p);
                }
                m_refTreeModel->erase((*iter));
                break;
            }
        }
    }

    g_torrent->remove_torrent(name);

    force_update = true;
    update_tooltip = true;
}

void
TreeView::on_popup_remove_torrent()
{
    if (selection) {
        Gtk::TreeModel::iterator iter = selection->get_selected();
        if (iter) {
            if(!(*iter)) return;

            /* Disregarding of whether we found a match or not,
             * the row should be removed seeing as it does no good */
            bf::path p((*iter)[col.path]);
            if (bf::exists(p)) {
                bf::remove(p);
            }
            std::string tmp_name(boost::lexical_cast<std::string>((*iter)[col.name]));

#ifdef GLIBMM_EXCEPTIONS_ENABLED
            try {
                m_refTreeModel->erase(*iter);
            } catch (...) {
                g_logger->error("unknown error!");
            }
#else
            m_refTreeModel->erase(*iter);
#endif

            remove_torrent(tmp_name, false);
        }
    }
}

void
TreeView::on_popup_pauseresume_torrent()
{
    if(selection) {
        Gtk::TreeModel::iterator iter = selection->get_selected();
        if(iter) {
            if(!(*iter)) return;

            g_torrent->toggle_pause(
                    boost::lexical_cast<std::string>((*iter)[col.name]));
        }
    }
}

void
TreeView::on_popup_recheck_torrent()
{
    if(selection) {
        Gtk::TreeModel::iterator iter = selection->get_selected();
        if(iter) {
            if(!(*iter)) return;

            g_torrent->force_recheck(
                    boost::lexical_cast<std::string>((*iter)[col.name]));
        }
    }
}

/** 
 * Forces the selected torrent to do another tracker request to
 * receive new peers
 **/
void
TreeView::on_popup_reannounce_torrent()
{
    if(selection) {
        Gtk::TreeModel::iterator iter = selection->get_selected();
        if(iter) {
            if(!(*iter)) return;

            g_torrent->force_reannounce(
                    boost::lexical_cast<std::string>((*iter)[col.name]));
        }
    }
}

/** 
 * Fetches the download directory from a torrent and opens up directory
 * with the appropriate file manager
 **/
void
TreeView::on_popup_goto_download_dir()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = get_selection();
    if(refSelection) {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter) {
            char path[512];
            sprintf(path, "%s", (std::string(boost::lexical_cast<std::string>((*iter)[col.download_path]))).c_str());
#ifdef DEBUG
            g_logger->messagef("opening %s", path);
#endif
            char *argv[] = {"xdg-open", path, 0};
            g_spawn_async(0, argv, 0, G_SPAWN_SEARCH_PATH, 0, 0, 0, 0);
        }
    }
}

/** 
 * Load all the torrents in the torrent directory
 * usually ~/.torium/torrents
 **/
bool
TreeView::loadTorrents()
{
    bf::path p(g_settings->torrent_dir);

    if(!bf::exists(p)) {
        if(!bf::create_directory(p)) {
            g_logger->error("Could not create torrent directory", true);
            return false;
        }
    }

    int fail = 0;
    if(!bf::is_directory(p)) {
        g_logger->error(g_settings->torrent_dir + " is not a directory!");
        return false;
    } else {
        bf::directory_iterator iter;

        /* loop through all files in the directory */
        for(bf::directory_iterator dir_iter(p); dir_iter != iter; ++dir_iter) {
            if(bf::is_directory(dir_iter->status())) {
#if BOOST_VERSION <= 103500
            } else if(bf::is_regular(dir_iter->status())) {
#else
            } else if(bf::is_regular_file(dir_iter->status())) {
#endif
                /* debugging
                std::cout << "      filename(): " << dir_iter->path().filename() << "\n"
                          << "     root_name(): " << dir_iter->path().root_name() << "\n"
                          << "root_directory(): " << dir_iter->path().root_directory() << "\n"
                          << "   file_string(): " << dir_iter->path().file_string() << "\n";
                */
                if(!add_torrent(dir_iter->path().file_string().c_str()))
                    fail++;
            }
        }
    }
    refreshTorrents();
    if(fail) {
        g_logger->warningf("Failed adding %d torrent(s)", fail);
        return false;
    }

    return true;
}

/** 
 * Whenever a row is doubleclicked, expand it
 * This will probably be removed when the torrent properties window is complete
 **/
void
TreeView::on_treeview_row_activated(const Gtk::TreeModel::Path &path,
        Gtk::TreeViewColumn* column)
{
    /*Gtk::TreeModel::iterator iter = m_refTreeModel->get_iter(path);

    if(iter) {
        Gtk::TreeModel::Row torrent = *iter;
        if(torrent[col.expanded]) {
            torrent[col.expanded] = false;
        } else {
            torrent[col.expanded] = true;
        }
        refreshTorrents();
    }
    */
    on_popup_pauseresume_torrent();
}

void
TreeView::on_selection_changed()
{
    Gtk::TreeModel::iterator iter = selection->get_selected();

    if(iter) {
        /*
        Gtk::TreeModel::Row torrent = *iter;
        g_logger->message(torrent[col.name] + " selected");
        */
    } else {
        /**/
    }
}

bool
TreeView::add_torrent(const char *path, const std::string download_path)
{
#ifdef DEBUG
    g_logger->messagef("TreeView::add_torrent(%s)", path);
#endif

    const char *name = g_torrent->add_torrent(path, download_path);
    if (strlen(name) > 0) {
        torrent_handle h;
        g_torrent->get_torrent(name, h);
        const torrent_info   t = h.get_torrent_info();

        Gtk::TreeModel::Row row;
        row = *(m_refTreeModel->append());

        const char *basepath = strrchr(path, DIR_SEP);
        std::string realpath(g_settings->torrent_dir + basepath);

        row[col.name]              = t.name();
        row[col.total_size]        = t.total_size();
        row[col.path]              = realpath;
        row[col.download_path]     = h.save_path().string();
        row[col.num_pieces]        = t.num_pieces();
        /* Get default value */
        if (strcmp(g_settings->get_option("action_on_completion"), "nothing") == 0) {
            row[col.action_on_completion] = 0;
        } else if (strcmp(g_settings->get_option("action_on_completion"), "remove") == 0) {
            row[col.action_on_completion] = 1;
        } else if (strcmp(g_settings->get_option("action_on_completion"), "stop") == 0) {
            row[col.action_on_completion] = 2;
        }

        update_torrent(row);
        force_update = true;
        update_tooltip = true;

        return true;
    }
#ifdef DEBUG
    g_logger->errorf("failed adding torrent treeview::add_torrent(%s)", path);
#endif

    return false;
}

void
TreeView::update_torrent(Gtk::TreeModel::Row torrent)
{
    if(torrent) {
        Glib::ustring tmp;
        char c[512];

        if(torrent[col.expanded]) {
            int tot_downloaded = torrent[col.downloaded]/1000000;
            int tot_size       = torrent[col.total_size]/1000000;
            int num_seeders    = torrent[col.num_seeders];
            int num_peers      = torrent[col.num_peers];

            snprintf(c, 512, "Downloaded %d of %d MB\n%d seeders | %d peers",
                     tot_downloaded, tot_size, num_seeders, num_peers);

            tmp = "<b>" + torrent[col.name] + "</b>\n" + c;

            torrent[col.str_name] = tmp;
        } else {
            Glib::ustring name = torrent[col.name];
            tmp = truncate(name, 40);

            torrent[col.str_name] = "<b>" + tmp+ "</b>";
        }
    }
}

TreeView::~TreeView()
{
    /* */
}
