#ifndef _TREEVIEW__H_
#define _TREEVIEW__H_

#include "properties.h"
#include "torrent.h"
#include "settings.h"
#include "config.h"
#include "logger.h"
#include "tools.h"

#include <gtkmm.h>

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"
#include "libtorrent/session.hpp"
#include "libtorrent/identify_client.hpp"
#include "libtorrent/alert_types.hpp"
#include "libtorrent/torrent_info.hpp"
#include "libtorrent/file.hpp"
#include "libtorrent/size_type.hpp"
#include "libtorrent/ip_filter.hpp"

using namespace libtorrent;
typedef std::multimap<std::string, libtorrent::torrent_handle> handles_t;

class TreeView : public Gtk::TreeView
{
  public:
    TreeView();
    virtual ~TreeView();

    void update_torrent(Gtk::TreeModel::Row torrent);

    bool loadTorrents();
    bool add_torrent(const char *path, const std::string download_path="");
    void refreshTorrents();
    void load_ip_filters();

    void expand_all();
    void collapse_all();
    void remove_torrent(const std::string name, bool row=true);
    void on_popup_remove_torrent();
    void on_popup_pauseresume_torrent();
    void on_popup_recheck_torrent();
    void on_popup_reannounce_torrent();
    void on_popup_properties();
    void on_popup_goto_download_dir();
    void check_torrent_alerts();

    bool force_update;
    bool update_tooltip;

  protected:
    Glib::RefPtr<Gtk::TreeView::Selection> selection;

    void updateTorrent(Gtk::TreeModel::Row torrent);
    virtual void on_treeview_row_activated(const Gtk::TreeModel::Path &path, Gtk::TreeViewColumn *column);
    virtual void on_selection_changed();
    virtual bool on_button_press_event(GdkEventButton *ev);
    void on_data_receive(
            const Glib::RefPtr<Gdk::DragContext> &context, int x, int y,
            const Gtk::SelectionData& selection_data, guint info, guint time);

    class Columns : public Gtk::TreeModel::ColumnRecord
    {
      public:
        Columns()
        {
            add(name);
            add(dspeed);
            add(uspeed);
            add(status);
            add(eta);
            add(percentage);

            add(path);

            add(str_name);
            add(dspeed_str);
            add(uspeed_str);
            add(status_num);

            add(expanded);
            add(downloaded);
            add(num_seeders);
            add(num_peers);
            add(total_size);
            add(download_path);
            add(num_pieces);
            add(stockid);

            add(action_on_completion);
        }

        Gtk::TreeModelColumn<Glib::ustring> name;
        Gtk::TreeModelColumn<std::string> eta;
        Gtk::TreeModelColumn<int> percentage;
        Gtk::TreeModelColumn<long> downloaded;
        Gtk::TreeModelColumn<int> num_seeders;
        Gtk::TreeModelColumn<int> num_peers;
        Gtk::TreeModelColumn<long> total_size;
        Gtk::TreeModelColumn<float> dspeed;
        Gtk::TreeModelColumn<float> uspeed;
        Gtk::TreeModelColumn<Glib::ustring> dspeed_str;
        Gtk::TreeModelColumn<Glib::ustring> uspeed_str;
        Gtk::TreeModelColumn<Glib::ustring> str_name;
        Gtk::TreeModelColumn<Glib::ustring> downloaded_str;
        Gtk::TreeModelColumn<std::string> path;
        Gtk::TreeModelColumn<Glib::ustring> status;
        Gtk::TreeModelColumn<int> status_num;
        Gtk::TreeModelColumn<bool> expanded;
        Gtk::TreeModelColumn<Glib::ustring> download_path;
        Gtk::TreeModelColumn<Gtk::StockID> stockid;
        Gtk::TreeModelColumn<int> num_pieces;
        Gtk::TreeModelColumn<int> action_on_completion;
    };

    Columns col;
    Glib::RefPtr<Gtk::ListStore> m_refTreeModel;
    Gtk::Menu popup_menu;

    void handle_completed(torrent_handle *h);
};
#endif
