#ifndef _WINDOW__H_
#define _WINDOW__H_

#include <gtkmm.h>
#include <errno.h>

#include "treeview.h"
#include "preferences.h"
#include "gui_create_torrent.h"

class MainWindow : public Gtk::Window
{
  public:
    MainWindow(std::vector<const char*> torrents, int tmp_x, int tmp_y, int tmp_width, int tmp_height);
    virtual ~MainWindow();

    int refresh_rate, elapsed, total_downloaded, total_uploaded;

    void save_stats();
    void monitor_dir();

    const char *pipe_file;
    const char *pid_file;
    int fd;
#ifndef WIN_32
    bool read_pipe();
    bool pipe_create(const char *pipename);
#endif

    TreeView treeview;

  protected:
    void on_menu_file_add_torrent();
    void on_menu_file_create_torrent();
    void on_menu_file_quit();
    void on_menu_edit_preferences();
    void on_menu_edit_load_ip_filters();
    void on_menu_help_contents();
    void on_menu_help_about();
    void on_menu_help_stats();

    bool on_timer();
    void on_status_icon_click();
    void on_status_icon_popup(guint button, guint32 activate_time);
    void save_torrents_status();

    /* A `cache' variable for the "torium %d/%d" part
     * of the torium tray icon tooltip */
    char tooltip_cache[32];
    int torrents_left;

    Preferences *preferences_window;

    /* This is for the menu bar, the UIManager handles the XML */
    Glib::RefPtr<Gtk::UIManager> m_refUIManager;
    Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
    Glib::RefPtr<Gtk::StatusIcon> status_icon;
    Gtk::Menu        status_icon_menu;
    Gtk::Widget     *pMenubar;

    Gtk::VBox m_Box;
    Gtk::ScrolledWindow m_ScrolledWindow;
    Gtk::Statusbar m_Statusbar;
};
#endif
